//****************************************************************************
//              Computing the Shannon entropy from recurrence plots
//                   Case of the Logistic map versus mu-parameter
//
//           C. Letellier, Physical Review Letters, 96, 254102, 2006
//
//                                          Last modification October 26, 2017
//****************************************************************************
#include <iostream>
#include <cmath>
#include <fstream>

using namespace std;

  const double amp = 0.000;                           // noise amplitude < 0.1

int main() 
{
    int i, j, k, l, m, N;
    int *count;
    const int Npt = 900;
    const int Ntrans = 250;
    const int Nbif = 300;                       // Number of parameter values
    double epsi = pow(3.0,0.5)*0.1;
    double Xmin = 1.0e+5;
    double Xmax = -1.0e+5;
    double mumin = 3.0;                            // Range for the mu-values
    double mumax = 3.999;
    double dmu = (mumax - mumin)/double(Nbif);
    double *x, mu, noise, Dis, Tr, Sh, Prob;
    double R[Npt][Npt];                             // Recurrence plot matrix

    x = new double[Npt];
    count = new int[Npt];
//***************************************************************************
    ofstream data_12("shannon.dat");

    mu = mumin;
    x[0] = 0.8;
    for (i = 0; i < Npt - 1; i++) 
    {
      noise = amp*(drand48() - 0.5);
      if ((x[i] + noise) >= 0.0 && (x[i] + noise) < 1.0) 
      {
        x[i] += noise;
      }
      x[i+1] = mu*x[i]*(1.0 - x[i]);
    }
    x[0] = x[Npt - 1];
    cout << mu << endl;
    for (mu = mumin; mu < mumax; mu += dmu) 
    {
      cout << mu << endl;
      for (i = 0; i < Ntrans; i++) 
      {
        noise = amp*drand48();
        if ((x[i] + noise) > 0.0 && (x[i] + noise) < 1.0) 
	{
          x[i] += noise;
        }
        x[i+1] = mu*x[i]*(1.0 - x[i]);
      }
      x[0] = x[Ntrans];
      for (i = 0; i < Npt - 1; i++) 
      {
        noise = amp*drand48();
        if ((x[i] + noise) > 0.0 && (x[i] + noise) < 1.0) 
	{
          x[i] += noise;
        }
        x[i+1] = mu*x[i]*(1.0 - x[i]);
        Xmin = min(Xmin, x[i+1]);
        Xmax = max(Xmax, x[i+1]);
      }
      epsi = pow(3.0,0.5)*(Xmax - Xmin)/10.0;
      if (epsi < 0.001) 
      {
        Sh = 0.0;
      } 
      else 
      {
        for (i = 0; i < Npt; i++) 
	{
          for (j = 0; j < Npt; j++) 
	  {
            R[i][j] = 0.0;
          }
        }
        for (i = 0; i < Npt - 2; i++) 
	{
          for (j = 0; j < Npt - 2; j++) 
	  {
            Dis = pow((pow(x[j] - x[i], 2) + pow(x[j+1] - x[i+1],2) 
				          + pow(x[j+2] - x[i+2], 2)),0.5);
            if (Dis <= epsi) 
	    {
              R[i][j] = 1.0;
            }
          }
        }
        for (i = 0; i < Npt; i++) 
	{
          count[i] = 0;                // Initialisation of the probabilities
        }
        for (i = 1; i < Npt - 1; i++) 
	{
          m = 1;
          for (j = 0; j < Npt - i; j++) 
	  {
            if (R[i+j - 1][j] < 0.5) 
	    {
              m++; 
            } 
	    else 
	    {
              count[m]++;
              m = 1;
            }
          }
        }
        N = 0;
        for (i = 0; i < Npt; i++) 
        {
          N += count[i];
        }
        Sh = 0.0;                             // Computing the probabilities
        for (i = 0; i < Npt; i++) 
        {
          if (count[i] > 0) 
          {
            Prob = double(count[i])/double(N);
            Sh -= Prob*log(Prob);
          }
        }
      }
      data_12 << mu << " " << Sh << endl;
    }
    data_12.close();
    system("xmgrace shannon.dat -param shalogis.par");
    return 0;
}
